## R-code for estimating all models associated to the following manuscript:
## 
## "The question of human mortality plateau: 
## Contrasting insights by longevity pioneers"
## Published by Demographic Research


## Inspired by the R codes provided by Francesco Lagona
## for the paper by Barbi et al. (2018) entitled 
## "The plateau of human mortality: Demography of longevity pioneers". Science
## !!!! Please note that the original website below is no longer available !!!!
## scienzepolitiche.uniroma3.it/flagona/publications-en 

## R-version used: R 3.6.3

## Required packages: 
## - survival
## - flexsurv 
## - HMDHFDplus

## Required external file: 
## - cdfDTmodified.R

## Clean the workspace from all objects
rm(list = ls())
## Get plots on a different screen
options(device="X11")
## Print a large number of decimals
options(scipen=20)
## Set up the directory where .R is saved (RStudio command)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

## Load useful packages:
library(survival)
## for the functions Surv(): creating a survival object
library(flexsurv)
## for the function flexsurvreg(): estimating parametric models for time-to-event data
library(HMDHFDplus)
## for the function readHMDweb(): extracting data directly from the Human Mortality Database website

## Load the function cdfDTmodified()
source("cdfDTmodified.R")
## Modified version of the function cdfDT in SurvTrunc
## to extract confidence intervals of the cumulative hazard
## from the internally bootstrap algorithm

## Data are taken from International Database on Longevity 
## for French birth cohorts born 1883-1901, 
## selecting individuals that were born and died in Metropolitan France

## Description of data used in this program:
## object-name: datafr
## object-mode: list
## object-class: data.frame
## size: 3789 x 7
## colnames: 
##   Sex: 0 (Females) / 1 (Males)
##   Cohort.base: year of birth - baseline cohort (1891)
##   Status: status indicator (1 for all)
##   Age.at.exit: the age at which individual die
##   Duration: length of life from age 105
##   Cohort: year of birth
##   Rtrunc: individual-specific right-truncation time assuming 31/12/2017 as last day with retrospective ascertainment 


## Table 1
tab <- table(datafr$Cohort, datafr$Sex)
tab <- rbind(tab, colSums(tab))
colnames(tab) <- c("Females", "Males")
tab

## Estimating models presented in the manuscript:

## Create a common survival object for all models
SurvFr <- Surv(datafr$Duration, datafr$Status)

#### EXPONENTIAL baseline ####
## Model without covariates
mod.const <- flexsurvreg(SurvFr ~ 1, data=datafr,
                         rtrunc = datafr$Rtrunc,
                         dist = "exponential")
## Model with sex alone
mod.const.sex <- flexsurvreg(SurvFr ~ Sex, data=datafr, 
                             rtrunc = datafr$Rtrunc,
                             dist = "exponential")
## Model with cohort alone
mod.const.cohort <- flexsurvreg(SurvFr ~ Cohort.base, data=datafr, 
                                rtrunc = datafr$Rtrunc,
                                dist = "exponential")
## Model with cohort and sex
mod.const.sex.cohort <- flexsurvreg(SurvFr ~ Sex + Cohort.base, data=datafr, 
                                    rtrunc = datafr$Rtrunc,
                                    dist = "exponential")

#### GOMPERTZ baseline ####
## Model without covariates
mod.gomp <- flexsurvreg(SurvFr ~ 1, data=datafr,
                        rtrunc = datafr$Rtrunc,
                        dist="gompertz")
## Model with sex alone
mod.gomp.sex <- flexsurvreg(SurvFr ~ Sex, data=datafr, 
                            rtrunc = datafr$Rtrunc,
                            dist = "gompertz")
## Model with cohort alone
mod.gomp.cohort <- flexsurvreg(SurvFr ~ Cohort.base, data=datafr, 
                               rtrunc = datafr$Rtrunc,
                               dist = "gompertz")
## Model with cohort and sex
mod.gomp.sex.cohort <- flexsurvreg(SurvFr ~ Sex + Cohort.base, data=datafr, 
                                   rtrunc = datafr$Rtrunc,
                                   dist = "gompertz")

## All models in a single object
all.mod <- list(mod.const, mod.const.sex, mod.const.cohort, mod.const.sex.cohort,
                mod.gomp, mod.gomp.sex, mod.gomp.cohort, mod.gomp.sex.cohort)
names(all.mod) <- c("const", "const+sex", "const+cohort", "const+sex+cohort",
                    "gomp", "gomp+sex", "gomp+cohort", "gomp+sex+cohort")

### Extract log-likelihood and AIC
all.loglik <- unlist(lapply(all.mod, logLik))
all.AIC <- unlist(lapply(all.mod, AIC))

## Test significance of Gompertz slope in all model specifications
# Models without covariate
chisq <- -2*(all.loglik["const"] - all.loglik["gomp"])
pchisq(chisq,1,lower.tail = FALSE)

# Models with gender effect
chisq <- -2*(all.loglik["const+sex"] - all.loglik["gomp+sex"])
pchisq(chisq,1,lower.tail = FALSE)

# Models with cohort effect
chisq <- -2*(all.loglik["const+cohort"] - all.loglik["gomp+cohort"])
pchisq(chisq,1,lower.tail = FALSE)

# Models with gender and cohort effects
chisq <- -2*(all.loglik["const+sex+cohort"] - all.loglik["gomp+sex+cohort"])
pchisq(chisq,1,lower.tail = FALSE)


## Test significance of Gompertz slope; b must be non-negative,
## hence the one-tailed hypothesis test
# Models without covariate
chisq <- -2*(all.loglik["const"] - all.loglik["gomp"])
pchisq(chisq,1,lower.tail = FALSE)
chisq.const <- 1/2*pchisq(chisq,1,lower.tail = FALSE) 

# Models with gender effect
chisq <- -2*(all.loglik["const+sex"] - all.loglik["gomp+sex"])
pchisq(chisq,1,lower.tail = FALSE)
chisq.sex <- 1/2*pchisq(chisq,1,lower.tail = FALSE) 

# Models with cohort effect
chisq <- -2*(all.loglik["const+cohort"] - all.loglik["gomp+cohort"])
pchisq(chisq,1,lower.tail = FALSE)
chisq.cohort <- 1/2*pchisq(chisq,1,lower.tail = FALSE) 

# Models with sex+cohort effects
chisq <- -2*(all.loglik["const+sex+cohort"] - all.loglik["gomp+sex+cohort"])
pchisq(chisq,1,lower.tail = FALSE)
chisq.sexNcohort <- 1/2*pchisq(chisq,1,lower.tail = FALSE) 


## Estimates and 95% Confidence intervals for all models
## "rate" represents the coefficient "a" for both Constant and Gompertz
## "shape" represents the coefficient "b" for Gompertz
Estim <- sapply(all.mod, function(f) f$res[,1:3])


## FIGURE 1 

## Compute the nonparametric MLE for the baseline hazard
## using the algorithm by Shen (2010)
## i.e. hazard for all females regardless of the cohort 
## (which is non significant in the selected model)
datafrF <- subset(datafr, datafr$Sex==0)
datafrM <- subset(datafr, datafr$Sex==1)

## ~8 minutes, with default 200 bootstrap samples
tim1 <- Sys.time() 
fitST <- cdfDTmodified(y=datafrF$Duration, 
                       l=rep(0, nrow(datafrF)),
                       r=datafrF$Rtrunc, boot=TRUE)
tim2 <- Sys.time()
(tim.ST <- tim2-tim1)


## Cumulative hazard functions for parametric models
# Gompertz hazard
H.GOMP <- function(a,b, x){
  num <- a*(exp(b*x)-1)
  den <- b
  H <- num/den
  return(H)
}
# Exponential hazard
H.EXP <- function(lambda, x){
  H <- lambda * x
  return(H)
}

## Extract the correct parameters for females (baseline hazard)
## from models with only sex as covariate
## which is the optimal models for both baseline specifications
lambda.hat <- all.mod$`const+sex`$res[1,1]
a.hat <- all.mod$`gomp+sex`$res[2,1]
b.hat <- all.mod$`gomp+sex`$res[1,1]

## Evaluate over the range of observed durations, 
## cumulative hazard for Gompertz and Exponential-Constant baselines
times <- seq(min(datafrF$Duration), max(datafrF$Duration), length=100)
H0.gomp <- H.GOMP(a=a.hat, b=b.hat, x=times)
H0.const <- H.EXP(lambda=lambda.hat, x=times)

## Plotting 
plot(fitST$time, fitST$H, xlab="Age (years)", ylab="Cumulative hazard",
     axes=FALSE, lty=1, lwd=2, las=0, ylim=c(0,8.2), xlim=c(0,9.1), col="red",
     t="l")
lines(fitST$time, fitST$CI.lower.H, col=2, lwd=2, lty=2)
lines(fitST$time, fitST$CI.upper.H, col=2, lwd=2, lty=2)
ticks.x <- seq(0,11,1)
ticks.y <- seq(0,8,1)
axis(side=1,at=ticks.x,labels=105:116)
axis(side=2,at=ticks.y,las=1)
lines(times,H0.gomp,lty=1,lwd=2,col="blue")
lines(times,H0.const,lty=1,lwd=2,col="darkgreen")
legend('topleft',c("NPMLE estimates & 95% CIs",
                   "Under Gompertz hazard",
                   "Under constant hazard"),
       col=c("red","blue","darkgreen"),
       lty=c(1,1,1),lwd=c(2,2,2),bty="n",cex=1.1)



## FIGURE 2

## Loading exposures from HMD 
Ehmd <- readHMDweb(CNTRY = "FRATNP", 
                   item = "cExposures_1x1",
                   username = "",
                   password = "",
                   fixup = TRUE)
## Loading rates from HMD
Mhmd <- readHMDweb(CNTRY = "FRATNP", 
                   item = "cMx_1x1", 
                   username = "",
                   password = "",
                   fixup = TRUE)
## Note that in the two previous steps
## if arguments "username" and "password" are not
## provided, you will be asked to type them in  

## Ages and cohorts for selecting the appropriate HMD data
ages <- 90:104
cohorts <- sort(unique(datafr$Cohort))

## Females
## Selecting cohorts and ages
E0 <- subset(Ehmd, Year%in%cohorts & Age%in%ages, "Female")
M0 <- subset(Mhmd, Year%in%cohorts & Age%in%ages, "Female")
E1 <- matrix(E0[,1], length(ages))
M1 <- matrix(M0[,1], length(ages))
## Summing up all cohorts since this variable is not selected
eF <- rowSums(E1)
dF <- rowSums(M1*E1)

## Males
## Selecting cohorts and ages
E0 <- subset(Ehmd, Year%in%cohorts & Age%in%ages, "Male")
M0 <- subset(Mhmd, Year%in%cohorts & Age%in%ages, "Male")
E1 <- matrix(E0[,1], length(ages))
M1 <- matrix(M0[,1], length(ages))
## Summing up all cohorts since this variable is not selected
eM <- rowSums(E1)
dM <- rowSums(M1*E1)

## Simple function to compute confidence interval 
## for rates assuming exact Poisson confidence limits 
## for the estimated rate
CI.rate <- function(d, e, conf.level = 0.95){
  ## alpha/2
  alpha.2 <- 1 - ((1 - conf.level)/2) 
  ## rate
  mx <- d/e
  ## Chi-square values
  chi2.low <- qchisq(p=1 - alpha.2, df=2*d)
  chi2.up  <- qchisq(p=alpha.2, df=2*(d + 1))
  ## Lower and upper limit
  low <- chi2.low/2/e
  up <- chi2.up/2/e
  ## Outcome
  out <- data.frame(mx = mx, low = low, up = up)
  return(out)
}

## Rates and 95% CI for both females and males 
CIf <- CI.rate(dF, eF)
CIm <- CI.rate(dM, eM)

## Computing annual death rates from individual data
## disregarding right truncation scheme

## Set up the length of interval (one year)
delta <- 1
## and minimum
min <- delta

## FEMALES
## Maximum
maxF <- max(datafrF$Duration)+delta
## Breaks for the splitting
taujF <- seq(min, maxF, delta)
## Splitting individual observations&
datafrFsplit <- survSplit(data=datafrF, cut=taujF,
                          end="Duration", 
                          event="Status", 
                          episode="interval")
## Adding individual-interval exposures times
datafrFsplit$exp.time <- datafrFsplit$Duration-datafrFsplit$tstart
## Forcing interval to be factors
datafrFsplit$interval <- as.factor(datafrFsplit$interval)
## Aggregating splited individual data, 
## here computing total number of event and total exposure times for each interval
datafrFpy <- pyears(Surv(tstart, Duration, Status) ~ interval,
                    data = datafrFsplit, scale=1)
## Creating the final aggregated dataset
datafrFagg <- data.frame(interval=unique(datafrFsplit$interval))
datafrFagg$event <- datafrFpy$event
datafrFagg$expo <- datafrFpy$pyears
## Fitting a piece-wise constant hazard model: estimating death log-rates
fitF <- glm(event ~ -1 + interval, offset = log(expo), family=poisson, data=datafrFagg)
## Computing confidence intervals for death rates (delta method)
V <- vcov(fitF)
gr <- diag(exp(fitF$coef))
tgVg <- t(gr) %*% V %*% gr
se.hF <- sqrt(diag(tgVg))
h.hatF <- exp(fitF$coeff)
h.upF <- h.hatF + 1.96*se.hF
h.lowF <- h.hatF - 1.96*se.hF

## MALES
## Same comments as for females above
maxM <- max(datafrM$Duration)+delta
taujM <- seq(min, maxM, delta)
datafrMsplit <- survSplit(data=datafrM, cut=taujM,
                          end="Duration", 
                          event="Status", 
                          episode="interval")
datafrMsplit$exp.time <- datafrMsplit$Duration-datafrMsplit$tstart
datafrMsplit$interval <- as.factor(datafrMsplit$interval)
datafrMpy <- pyears(Surv(tstart, Duration, Status) ~ interval,
                    data = datafrMsplit, scale=1)
datafrMagg <- data.frame(interval=unique(datafrMsplit$interval))
datafrMagg$event <- datafrMpy$event
datafrMagg$expo <- datafrMpy$pyears
fitM <- glm(event ~ -1 + interval, offset = log(expo), family=poisson, data=datafrMagg)
V <- vcov(fitM)
gr <- diag(exp(fitM$coef))
tgVg <- t(gr) %*% V %*% gr
se.hM <- sqrt(diag(tgVg))
h.hatM <- exp(fitM$coeff)
h.upM <- h.hatM + 1.96*se.hM
h.lowM <- h.hatM - 1.96*se.hM

## Plotting FIGURE 2

x11()
par(mfrow=c(1,2),mar=c(4.5, 5, 1, 0.7))

## Plotting females
plot(ages+0.5,CIf$mx,log="y",pch=20, cex=1.2,
     xlim=c(90,115),ylim=c(0.2,1.4),
     xlab="",ylab="",axes=FALSE,font=2,font.lab=2)
box(lwd=3)
ticks.x <- seq(90,115,5)
ticks.y <- seq(0.2,2,0.2)
axis(side=1,at=ticks.x,cex.axis=1.3,font=2,lwd.ticks=2)
axis(side=2,at=ticks.y,cex.axis=1.3,font=2,las=1,lwd.ticks=2)
mtext(text="Age (years)",side=1,line=2.6,cex=1.3,font=2)
mtext(text="Hazard (log scale)", side=2, line= 3, cex=1.3, las=0,font=2)
arrows(ages+0.5,CIf$low,ages+0.5,CIf$up,
       length=0.05,angle=90,code=3,lwd=3)
abline(v=105,col="grey",lty=3,lwd=1.5)
## Adding Gompertz outcomes
VCgomp <- all.mod$`gomp+sex`$cov
g1a.gompF <- exp(b.hat*times)
g1b.gompF <- a.hat*times*exp(b.hat*times)
g1beta.gompF <- rep(0, length(times))
gr.gompF <- rbind(g1a.gompF, g1b.gompF, g1beta.gompF)
VgompF <- t(gr.gompF) %*% VCgomp %*% gr.gompF
SEgompF <- sqrt(diag(VgompF))

h.gompF <- a.hat*exp(b.hat*times)
h.gompFlow <- h.gompF - qnorm(0.975)*SEgompF
h.gompFup  <- h.gompF + qnorm(0.975)*SEgompF
polygon(c(times+105,rev(times+105)),
        c(h.gompFlow,rev(h.gompFup)),
        col="thistle1",border=NA)
lines(times+105, h.gompF, col="darkred", lwd=2)
abline(h=h.gompF[1],lty=3,col="darkred",lwd=2)

text(x=108.5,h.gompF[times==5],
     "h(105)=0.581",pos=1,col="darkred",cex=1.6,font=2)


## Adding Exponential outcomes
VCconst <- all.mod$`const+sex`$cov
g1lambda.constF <- rep(1, length(times))
g1beta.constF <- rep(0, length(times))
gr.constF <- rbind(g1lambda.constF, g1beta.constF)
VconstF <- t(gr.constF) %*% VCconst %*% gr.constF
SEconstF <- sqrt(diag(VconstF))

h.constF <- rep(lambda.hat, length(times))
h.constFlow <- h.constF - qnorm(0.975)*SEconstF
h.constFup  <- h.constF + qnorm(0.975)*SEconstF

lines(times+105, h.constFlow, col="blue",lwd=2, lty=4)
lines(times+105, h.constFup, col="blue",lwd=2, lty=4)
lines(times+105, h.constF, col="blue",lwd=3)
abline(h=lambda.hat,lty=3,col="blue",lwd=3)

text(x=101.5,h.constF[times==5],
     "h(105)=0.638",pos=3,col="blue",cex=1.6,font=2)

## Adding NPMLE for the hazard & 95% CI, i.e. death rates
points(taujF-delta/2+105, h.hatF, pch=20, cex=1, col="grey70")
arrows(taujF-delta/2+105,h.lowF,
       taujF-delta/2+105,h.upF,
       length=0.05,angle=90,code=3,lwd=3, col="grey70")

arrows(97,0.2,104,0.2,length=0.1,angle=30,code=1,lwd=2.8)
arrows(104,0.2,104.8,0.2,length=0.08,angle=90,code=2,lwd=2.8)
text(x=102.8,y=0.225,"HMD data",pos=1,cex=1.4)

arrows(105.2,0.2,112,0.2,length=0.08,angle=90,code=1,lwd=2.8)
arrows(110,0.2,112.5,0.2,length=0.1,angle=30,code=2,lwd=2.8)
text(x=107,y=0.24,"Validated",pos=1,cex=1.4)
text(x=108,y=0.225,"individual data",pos=1,cex=1.4)

legend(x=88,y=1.1,
       c("Gompertz PH",
         "Constant PH"),
       col=c("darkred","blue"),
       lwd=c(3,3),x.intersp = 0.2,bty="n",cex=1.4,seg.len=0.6)

## Plotting males
plot(ages+0.5,CIm$mx,log="y",pch=20, cex=1.2,
     xlim=c(90.5,115),ylim=c(0.2,1.4),
     xlab="",ylab="",axes=FALSE,font=2,font.lab=2)
box(lwd=3)
ticks.x <- seq(90,115,5)
ticks.y <- seq(0.2,2,0.2)
axis(side=1,at=ticks.x,cex.axis=1.3,font=2,lwd.ticks=2)
axis(side=2,at=ticks.y,cex.axis=1.3,font=2,las=1,lwd.ticks=2,labels=FALSE)
mtext(text="Age (years)",side=1,line=2.6,cex=1.3,font=2)
# mtext(text="Hazard (log scale)", side=2, line= 3, cex=1.3, las=0,font=2)
arrows(ages+0.5,CIm$low,ages+0.5,CIm$up,
       length=0.05,angle=90,code=3,lwd=3)
abline(v=105,col="grey",lty=3,lwd=1.5)
## Adding Gompertz outcomes
timesM <- seq(min(datafrM$Duration), max(datafrM$Duration), length=100)
betaG <- all.mod$`gomp+sex`$coefficients[3]

g1a.gompM <- exp(b.hat*timesM)*exp(betaG)
g1b.gompM <- a.hat*timesM*exp(b.hat*timesM)*exp(betaG)
g1beta.gompM <- a.hat*exp(b.hat*timesM)*exp(betaG)
gr.gompM <- rbind(g1a.gompM, g1b.gompM, g1beta.gompM)
VgompM <- t(gr.gompM) %*% VCgomp %*% gr.gompM
SEgompM <- sqrt(diag(VgompM))

h.gompM <- a.hat*exp(b.hat*timesM)*exp(betaG)
h.gompMlow <- h.gompM - qnorm(0.975)*SEgompM
h.gompMup  <- h.gompM + qnorm(0.975)*SEgompM
polygon(c(timesM+105,rev(timesM+105)),
        c(h.gompMlow,rev(h.gompMup)),
        col="thistle1",border=NA)
lines(timesM+105, h.gompM, col="darkred", lwd=2)
abline(h=h.gompM[1],lty=3,col="darkred",lwd=2)

## Adding Exponential outcomes
betaE <- all.mod$`const+sex`$coefficients[2]
g1lambda.constM <- exp(betaE*rep(1, length(timesM)))
g1beta.constM <- lambda.hat*exp(betaE*rep(1, length(timesM)))
gr.constM <- rbind(g1lambda.constM, g1beta.constM)
VconstM <- t(gr.constM) %*% VCconst %*% gr.constM
SEconstM <- sqrt(diag(VconstM))

h.constM <- rep(lambda.hat, length(timesM))*exp(betaE)
h.constMlow <- h.constM - qnorm(0.975)*SEconstM
h.constMup  <- h.constM + qnorm(0.975)*SEconstM

lines(timesM+105, h.constMlow, col="blue",lwd=2, lty=4)
lines(timesM+105, h.constMup, col="blue",lwd=2, lty=4)
lines(timesM+105, h.constM, col="blue",lwd=3)
abline(h=lambda.hat*exp(betaE),lty=3,col="blue",lwd=2)

## Adding NPMLE for the hazard & 95% CI, i.e. death rates
points(taujM-delta/2+105,h.hatM, pch=20, cex=1, col="grey70")
arrows(taujM-delta/2+105,h.lowM,
       taujM-delta/2+105,h.upM,
       length=0.05,angle=90,code=3,lwd=3, col="grey70")

arrows(97,0.2,104,0.2,length=0.1,angle=30,code=1,lwd=2.8)
arrows(104,0.2,104.8,0.2,length=0.08,angle=90,code=2,lwd=2.8)
text(x=102.8,y=0.225,"HMD data",pos=1,cex=1.4)

arrows(105.2,0.2,112,0.2,length=0.08,angle=90,code=1,lwd=2.8)
arrows(110,0.2,112.5,0.2,length=0.1,angle=30,code=2,lwd=2.8)
text(x=107,y=0.24,"Validated",pos=1,cex=1.4)
text(x=108.0,y=0.225,"individual data",pos=1,cex=1.4)

legend(x=88,y=1.1,
       c("Gompertz PH",
         "Constant PH"),
       col=c("darkred","blue"),
       lwd=c(3,3),x.intersp = 0.2,bty="n",cex=1.4,seg.len=0.6)





